package jdbernard.timestamper.core;

import java.text.SimpleDateFormat;

import java.util.Collection;
import java.util.Date;
import java.util.Iterator;
import java.util.TreeSet;

/**
 * @author Jonathan Bernard {@literal <jdbernard@gmail.com>}
 * A Timeline object represents a series of markers at specific points in time.
 * The markers have a name or symbol (the 'mark') and notes associated with that
 * mark.
 */
public class Timeline implements Iterable<TimelineMarker> {

    public static SimpleDateFormat shortFormat = new SimpleDateFormat("HH:mm:ss");
    public static SimpleDateFormat longFormat = new SimpleDateFormat("EEE MMM dd HH:mm:ss zzz yyyy");
    private TreeSet<TimelineMarker> timelineList;

    public Timeline() {
        timelineList = new TreeSet<TimelineMarker>();
    }

    public void addMarker(Date timestamp, String name, String notes) {
        timelineList.add(new TimelineMarker(timestamp, name, notes));
    }

    public String getLastName(Date timestamp) {
        TimelineMarker lastMarker = getLastMarker(timestamp);
        return (lastMarker == null ?
            "No previous marker." :
            lastMarker.getMark());

    }

    public String getLastNotes(Date timestamp) {
        TimelineMarker lastMarker = getLastMarker(timestamp);
        return (lastMarker == null ?
            "No previous marker." :
            lastMarker.getNotes());
    }

    public TimelineMarker getLastMarker(Date timestamp) {
        TimelineMarker lastMarker = null;
        for (TimelineMarker tm : timelineList) {
            if (tm.getTimestamp().after(timestamp))
                break;
            lastMarker = tm;
        }

        return lastMarker;
    }

    public void removeMarker(TimelineMarker marker) {
        timelineList.remove(marker);
    }

    public Iterator<TimelineMarker> iterator() {
        return timelineList.iterator();
    }

    /**
     * Return the difference of the this timeline relative to another timeline.
     * More specifically, return the set of all
     * {@link jdbernard.timestamper.core.TimelineMarker}s that are present in
     * the <b>this</b> timeline but not present in the given timeline.
     * timeline.
     * @param t
     * @return A collection representing the TimelineMarkers present in
     * <b>this</b> timeline but not in the given timeline.
     */
    public Collection<TimelineMarker> difference(Timeline t) {
        TreeSet<TimelineMarker> difference = new TreeSet<TimelineMarker>();

        for (TimelineMarker tm : timelineList) {
            if (!t.timelineList.contains(tm))
                difference.add(tm);
        }

        return difference;
    }

    public void addAll(Timeline t) {
        for (TimelineMarker tm : t) {
            if (!timelineList.contains(tm))
                timelineList.add(tm);
        }
    }

    public void addAll(Collection<TimelineMarker> c) {
        timelineList.addAll(c);
    }
}
